#!/usr/bin/env python
"""
_LoadFromID_

Oracle implementation of Subscription.LoadFromID
"""




from WMCore.WMBS.MySQL.Subscriptions.LoadFromID import LoadFromID as LoadFromIDMySQL

class LoadFromID(LoadFromIDMySQL):
    pass
