"""
_WorkUnit.ExistsByTaskFileLumi_

Oracle implementation of WorkUnit.ExistsByTaskFileLumi
"""

from __future__ import absolute_import, division, print_function

from WMCore.WMBS.MySQL.WorkUnit.ExistsByTaskFileLumi import ExistsByTaskFileLumi as ExistsByTaskFileLumiMySQL


class ExistsByTaskFileLumi(ExistsByTaskFileLumiMySQL):
    """
    _WorkUnit.ExistsByTaskFileLumi_

    Oracle implementation of WorkUnit.ExistsByTaskFileLumi
    """

    pass
