"""
_WorkUnit.GetByTaskFileLumi_

Oracle implementation of WorkUnit.GetByTaskFileLumi
"""

from __future__ import absolute_import, division, print_function

from WMCore.WMBS.MySQL.WorkUnit.GetByTaskFileLumi import GetByTaskFileLumi as GetByTaskFileLumiMySQL


class GetByTaskFileLumi(GetByTaskFileLumiMySQL):
    """
    _WorkUnit.GetByTaskFileLumi_

    Oracle implementation of WorkUnit.GetByTaskFileLumi
    """

    pass
