#!/usr/bin/env python
"""
_GetParentAndGrandParentInfo_

Oracle implementation of Files.GetParentAndGrandParentInfo
"""
from __future__ import division

from WMCore.WMBS.MySQL.Files.GetParentAndGrandParentInfo import \
    GetParentAndGrandParentInfo as GetParentAndGrandParentInfoMySQL


class GetParentAndGrandParentInfo(GetParentAndGrandParentInfoMySQL):
    pass
