#!/usr/bin/env python
"""
_InsertOutput_

Oracle implementation of Workflow.InsertOutput
"""

from WMCore.WMBS.MySQL.Workflow.InsertOutput import InsertOutput as InsertOutputMySQL

class InsertOutput(InsertOutputMySQL):
    pass
