#!/usr/bin/env python
"""
_LoadFromNameAndTask_

Oracle implementation of Workflow.LoadFromNameAndTask

"""
from __future__ import division

from WMCore.WMBS.MySQL.Workflow.LoadFromNameAndTask import LoadFromNameAndTask \
    as LoadFromNameAndTaskMySQL


class LoadFromNameAndTask(LoadFromNameAndTaskMySQL):
    pass
